from abara_file_io.ini import read_ini, write_ini
from abara_file_io.json import read_json, write_json
from abara_file_io.text import read_text, write_text
from abara_file_io.toml import read_toml, write_toml
from abara_file_io.yaml import read_yaml, write_yaml

__all__ = [
    'read_ini',
    'read_json',
    'read_text',
    'read_toml',
    'read_yaml',
    'write_ini',
    'write_json',
    'write_text',
    'write_toml',
    'write_yaml',
]



from logging import WARNING, getLogger
from pathlib import Path
from textwrap import dedent

import pytest

log = getLogger()


@pytest.fixture(autouse=True)
def log_level_set() -> None:
    getLogger('chardet').setLevel(WARNING)


# test_str


@pytest.fixture(scope='session')
def sample_str() -> str:
    text = """\
    瑣　事

    　人生を幸福にするためには、日常の瑣事を愛さなければならぬ。雲の光、竹の戦ぎ、群雀の声、行人の顔、――あらゆる日常の瑣事のうちに無上の甘露味を感じなければならぬ。
    　人生を幸福にするためには？　しかし瑣事を愛するものは瑣事のために苦しまなければならぬ。庭前の古池に飛びこんだ蛙は百年の愁いを破ったであろう。が、古池を飛び出した蛙は百年の愁いを与えたかもしれない。いや、芭蕉の一生は享楽の一生であるとともに、誰の目にも受苦の一生である。我々も微妙に楽しむためには、やはりまた微妙に苦しまなければならぬ。
    　人生を幸福にするためには、日常の瑣事に苦しまなければならぬ。雲の光、竹の戦ぎ、群雀の声、行人の顔、――あらゆる日常の瑣事のうちに堕地獄の苦痛を感じなければならぬ。
    """  # noqa: E501
    return dedent(text)


@pytest.fixture(scope='session')
def create_sample_text_file(tmp_path_factory: pytest.TempPathFactory, sample_str: str) -> Path:
    dir_path = tmp_path_factory.mktemp('pytest')
    file_path = dir_path / 'sample_text.txt'
    file_path.touch()
    file_path.write_text(sample_str)
    return file_path


@pytest.fixture
def create_sample_text_files_multiple_encodings(
    tmp_path_factory: pytest.TempPathFactory, sample_str: str, request: pytest.FixtureRequest
) -> Path:
    dir_path: Path = tmp_path_factory.mktemp('pytest')
    file_path: Path = dir_path / f'sample_text_{request.param}.txt'
    file_path.touch()
    file_path.write_text(sample_str, encoding=str(request.param), newline='\n')
    return file_path


@pytest.fixture
def create_sample_text_encode_false_detection(
    tmp_path_factory: pytest.TempPathFactory,
) -> dict[str, str | Path]:
    dir_path: Path = tmp_path_factory.mktemp('pytest')
    file_path: Path = dir_path / 'sample_text_fail.txt'
    file_path.touch()

    data = 'あ'
    file_path.write_text(data, encoding='cp932', newline='\r\n')
    return {'data': data, 'path': file_path}


# test_dicts


@pytest.fixture
def sample_dicts(request: pytest.FixtureRequest) -> dict[str, dict | str | bool]:
    match request.param:
        case 1:
            return {
                'data': {'foo': 1, 'bar': 2.236, 'baz': 'three', 'qux': True},
                'name': 'flat_dict',
                'expected': True,
                'ini_expected': True,
            }
        case 2:
            return {
                'data': {
                    'section1': {'foo': 'one', 'bar': 2.828, 'baz': 'three', 'qux': True},
                    'section2': {'foo': 1.732, 'bar': 'two', 'baz': 3, 'qux': False},
                },
                'name': 'section_dict1',
                'expected': True,
                'ini_expected': True,
            }
        case 3:
            return {
                'data': {
                    'section1': {'foo': {'qux': 4}, 'bar': 'two', 'baz': 3.14},
                    'section2': {
                        'foo': 'one',
                        'bar': [1, 2, 3],
                        'baz': {'qux': 130, 'quux': 256},
                    },
                },
                'name': 'section_dict2',
                'expected': True,
                'ini_expected': False,
            }
        case _:
            return {'data': {}, 'name': 'Empty_Dict', 'expected': False, 'ini_expected': False}



from configparser import ConfigParser
from logging import getLogger
from os import PathLike
from pathlib import Path
from typing import IO, Any, cast

from abara_file_io.common_io_wrapper import (
    common_file_read_exception_handling,
    common_file_write_exception_handling,
)

log = getLogger(__name__)


type IniConfigValue = str | int | float | bool


def _restore_ini_config(input_str: str) -> IniConfigValue:
    """iniファイル化でstrに変換された値を元に戻す

    Args:
        input_str (str): iniから読み込んだ値

    Returns:
        IniConfigValue: 修正された値
    """
    try:
        return int(input_str)
    except ValueError:
        pass

    try:
        return float(input_str)
    except ValueError:
        pass

    if input_str == 'True':
        return True
    if input_str == 'False':
        return False

    return input_str


def _restore_ini_configs(input_dict: dict[str, str]) -> dict[str, IniConfigValue]:
    """辞書内のvalueをini化する前の元の型に復元する

    Args:
        input_dict (dict[str, str]): iniファイルから読み込んだ辞書

    Returns:
        dict[str, IniConfigValue]: 型を修正された辞書
    """
    return {key: _restore_ini_config(value) for key, value in input_dict.items()}


def read_ini(
    path: str | PathLike[str],
) -> dict[str, IniConfigValue] | dict[str, dict[str, IniConfigValue]]:
    """iniをファイルを読み込み、辞書に変換して出力する

    iniはstr以外を扱えないので、辞書に変換する時に自動的にint,float,boolをpythonの型に変換する
    ソースとなる辞書に文字列で'True'や'12.34'などを保存していた場合も、strやfloatに変換される

    Args:
        path (str | PathLike[str]): _description_

    Returns:
        dict[str, IniConfigValue] | dict[str, dict[str, IniConfigValue]]:
            IniConfigValueはiniに保存できるstr,int,float,boolの4種類のどれか
    """

    def read_ini_core(
        f: IO[Any],
    ) -> ConfigParser:
        config = ConfigParser()
        config.read_file(f)
        return config

    config = common_file_read_exception_handling(
        func=read_ini_core, return_empty_value=ConfigParser(), path=path
    )

    config_sections = config.sections()
    config_result: dict = {}
    if len(config_sections) > 1:
        for i in config_sections:
            config_result[i] = _restore_ini_configs(dict(config.items(i)))
    elif len(config_sections) == 1:
        config_result = _restore_ini_configs(dict(config.items(config_sections[0])))
    else:
        log.warning('iniファイルのセクションが存在しません')
        config_result = {}

    return config_result


def _correct_all_input_values(input_dict: dict) -> bool:
    """入力された辞書のvalueが全てIniConfig = str | int | float | bool であればTrueを返す

    Args:
        input_dict (dict): 判定する辞書

    Returns:
        bool: 全てが str | int | float | bool であればTrue
    """
    return all(isinstance(i, (str, int, float, bool)) for i in input_dict.values())


def _data_ini_convertible_is_decision(
    data: dict[str, IniConfigValue] | dict[str, dict[str, IniConfigValue]],
) -> ConfigParser:
    """入力されたデータをiniに変換できるか判定してconfigparserに書き込む

    Args:
        data (dict[str, IniConfigValue] | dict[str, dict[str, IniConfigValue]]):
            iniに書き込む辞書データ

    Returns:
        ConfigParser: 辞書の内容を格納したconfigparser
    """
    config = ConfigParser()

    data_values_all_dict_type: bool = all(isinstance(i, dict) for i in data.values())
    data_values_all_ini_config_type = all(
        _correct_all_input_values(i) for i in data.values() if isinstance(i, dict)
    )
    if len(data) == 0:
        log.warning('入力された内容が空の辞書です')

    elif data_values_all_dict_type and data_values_all_ini_config_type:
        log.debug('multi section')
        multi_section_data = cast('dict[str, dict[str, IniConfigValue]]', data)
        for i in multi_section_data:
            config.add_section(i)
            for key, value in multi_section_data[i].items():
                config[i][key] = str(value)

    elif _correct_all_input_values(data):
        log.debug('single section')
        config.add_section('configs')
        for key, value in data.items():
            config['configs'][key] = str(value)
    else:
        log.warning('入力された内容がini化できない形式です')

    return config


def write_ini(
    data: dict[str, IniConfigValue] | dict[str, dict[str, IniConfigValue]],
    path: str | PathLike[str],
) -> bool:
    """辞書をiniファイルとして保存する

    保存できる要素は IniConfigValue = str | int | float | bool の4種類
    それ以外のSequence型はiniでは保存できない
    Args:
        data (dict[str, IniConfigValue] | dict[str, dict[str, IniConfigValue]]):
            iniに書き込む辞書オブジェクト
        path (str | PathLike[str]): 保存するファイルパス、ファイル名の拡張子まで記入

    Returns:
        bool: ファイルの保存に成功したらTrue、失敗したらFalse
    """
    path = Path(path)

    config = _data_ini_convertible_is_decision(data)

    if len(config.sections()) == 0:
        return False

    def write_ini_core(
        config: object,
        f: IO[Any],
    ) -> None:
        if isinstance(config, ConfigParser):
            config.write(f)

    return common_file_write_exception_handling(func=write_ini_core, data=config, path=path)



import tomllib
from logging import getLogger
from os import PathLike
from typing import IO, Any

import tomli_w

from abara_file_io.common_io_wrapper import (
    common_file_read_exception_handling,
    common_file_write_exception_handling,
)

log = getLogger(__name__)


def read_toml(path: str | PathLike[str]) -> dict:
    """TOMLファイルを読み込む

        読み込みに失敗した場合は空の辞書を返す

    Args:
        path (Union[Path, str]): 読み込むファイルのパス

    Returns:
        Union[dict, None]:
            正確にはdictのインスタンスのruamel.yaml.comments.CommentedMap
    """

    def read_toml_core(
        f: IO[Any],
    ) -> dict:
        return tomllib.load(f)

    return common_file_read_exception_handling(
        func=read_toml_core, return_empty_value={}, path=path, mode='rb'
    )


def write_toml(data: dict, path: str | PathLike[str]) -> bool:
    """TOMLとして書き込む

        第一引数で受け取ったパスに、第二引数で受け取った内容をTOMLとして書き込む。
        ※tomli_wが必要
        tomli_wでの書き込みはコメントが消えるので注意
        保持したい場合はtomlkitを使うこと

    Args:
        data (dict): tomlに書き込む辞書オブジェクト
        path (str | PathLike): 保存するファイルパス、ファイル名の拡張子まで記入

    Returns:
        bool: ファイルの保存に成功したらTrue、失敗したらFalse
    """

    def write_toml_core(
        data: object,
        f: IO[Any],
    ) -> None:
        if isinstance(data, dict):
            tomli_w.dump(data, f)

    return common_file_write_exception_handling(
        func=write_toml_core, data=data, path=path, mode='wb'
    )



from logging import getLogger
from os import PathLike
from typing import IO, Any

from ruamel.yaml import YAML

from abara_file_io.common_io_wrapper import (
    common_file_read_exception_handling,
    common_file_write_exception_handling,
)

log = getLogger(__name__)


def read_yaml(path: str | PathLike[str]) -> dict:
    """YAMLファイルの読み込み

        リスト、もしくは辞書型の変数として読み込む
        読み込みに失敗した場合は空の辞書を返す

    Args:
        path (Union[Path, str]): 書き込むファイル名

    Returns:
        Union[dict, None]:
            正確にはdictのインスタンスのruamel.yaml.comments.CommentedMap
    """

    def read_yaml_core(
        f: IO[Any],
    ) -> dict:
        yaml = YAML()
        return yaml.load(f)

    return common_file_read_exception_handling(
        func=read_yaml_core, return_empty_value={}, path=path
    )


def write_yaml(data: list | dict, path: str | PathLike[str]) -> bool:
    """YAMLファイルとして出力する

        第一引数で受け取ったパスに、第二引数で受け取った内容をYAMLとして書き込む。

    Args:
        data (dict): yamlに書き込む辞書オブジェクト
        path (str | PathLike): 保存するファイルパス、ファイル名の拡張子まで記入

    Returns:
        bool: ファイルの保存に成功したらTrue、失敗したらFalse
    """

    def write_yaml_core(
        data: object,
        f: IO[Any],
    ) -> None:
        yaml.dump(data, f)

    yaml = YAML()
    yaml.indent(mapping=2, sequence=4, offset=2)

    return common_file_write_exception_handling(func=write_yaml_core, data=data, path=path)



from pathlib import Path
from typing import IO, Any

import pytest
from ruamel.yaml.parser import ParserError

from abara_file_io.common_io_wrapper import (
    common_file_read_exception_handling,
    common_file_write_exception_handling,
)


@pytest.mark.parametrize(
    'exception',
    [
        pytest.param(FileNotFoundError, id='FileNotFoundError'),
        pytest.param(PermissionError, id='PermissionError'),
        pytest.param(IsADirectoryError, id='IsADirectoryError'),
        pytest.param(OSError, id='OSError'),
        pytest.param(ParserError, id='ParserError'),
    ],
)
def test_common_file_read_exception_handling(exception: Exception, tmp_path: Path) -> None:
    def dummy_func(file: object) -> None:
        raise exception

    path = tmp_path / 'dummy_file.txt'
    path.touch()

    response = common_file_read_exception_handling(
        func=dummy_func, return_empty_value='', path=path
    )

    assert response == ''


@pytest.mark.parametrize(
    'exception',
    [
        pytest.param(PermissionError, id='PermissionError'),
        pytest.param(IsADirectoryError, id='IsADirectoryError'),
        pytest.param(OSError, id='OSError'),
    ],
)
def test_common_file_write_exception_handling(exception: Exception, tmp_path: Path) -> None:
    def dummy_func(file: object, f: IO[Any]) -> None:
        raise exception

    path = tmp_path / 'dummy_file.txt'

    data = {'key': 'value'}

    response = common_file_write_exception_handling(func=dummy_func, data=data, path=path)

    assert not response



from logging import getLogger
from pathlib import Path

import pytest

from abara_file_io import read_text, write_text

log = getLogger(__name__)


def test_read_text(create_sample_text_file: Path, sample_str: str) -> None:
    read_data = read_text(create_sample_text_file)
    assert isinstance(read_data, str)
    assert read_data == sample_str


def test_write_text(tmp_path: Path, sample_str: str) -> None:
    file_path = tmp_path / 'pytest' / 'pytest_str.txt'
    write_text(sample_str, file_path)
    read_data = file_path.read_text(encoding='utf_8')
    assert read_data == sample_str


@pytest.mark.parametrize(
    'create_sample_text_files_multiple_encodings',
    [('utf_8'), ('shift_jis'), ('utf_16'), ('euc_jp')],
    indirect=['create_sample_text_files_multiple_encodings'],
)
def test_read_text_multiple_encodings(
    create_sample_text_files_multiple_encodings: Path, sample_str: str
) -> None:
    read_data = read_text(create_sample_text_files_multiple_encodings)
    assert read_data == sample_str


def test_encode_false_detection(
    create_sample_text_encode_false_detection: dict[str, str | Path],
) -> None:
    read_data = read_text(create_sample_text_encode_false_detection['path'])
    assert read_data == create_sample_text_encode_false_detection['data']


def test_bat_file(tmp_path: Path, sample_str: str) -> None:
    file_path = tmp_path / 'pytest' / 'pytest_bat.bat'
    write_text(sample_str, file_path)
    read_data = read_text(file_path)
    assert read_data == sample_str



from collections.abc import Callable
from logging import getLogger
from os import PathLike
from pathlib import Path
from typing import IO, Any, Literal, TypeVar

from charset_normalizer import from_path
from ruamel.yaml.parser import ParserError

log = getLogger(__name__)


T = TypeVar('T', bound=object)


def _decision_encoding(func: Callable[[IO[Any]], T], path: Path) -> T | None:
    """charset_normalizerの文字コード判定を候補順に全て試行する

    Returns:
        T | None: 呼び出し時に設定した戻り値の型
    """
    results = from_path(path)

    for i in results:
        try:
            with path.open(mode='r', encoding=i.encoding) as f:
                return func(f)
        except UnicodeDecodeError:
            log.debug(f'文字コード{i.encoding}での読み込み試行失敗')
    return None


def common_file_read_exception_handling(
    func: Callable[[IO[Any]], T],
    return_empty_value: T,
    path: str | PathLike[str],
    *,
    mode: Literal['r', 'rb'] = 'r',
    encoding: str | None = 'utf_8',
) -> T:
    """ファイル読み込み時の汎用的な例外処理をするラッパー関数

    Args:
        func (Callable[[IO[Any]], T]): openしたファイルの読み込みをする関数
        return_empty_value (T): 読み込み処理の失敗時に戻る値。これによって戻り値の型も決定する
        path (str | PathLike[str]): 開くファイルのパス
        mode (Literal['r', 'rb';], optional): 読み込むファイルを開く時のmode. Defaults to 'r'.
        encoding (str | None): 読み込む時の文字コード Defaults to 'utf_8'.

    Returns:
        T: 呼び出し時にreturn_empty_valueで設定した戻り値の型
    """
    p = Path(path)

    if mode == 'rb':
        encoding = None

    try:
        with p.open(mode=mode, encoding=encoding) as f:
            read_data: T = func(f)
    except UnicodeDecodeError:
        result = _decision_encoding(func=func, path=p)

        if result is not None:
            return result

        log.warning(
            '読み込もうとしたファイルの文字コードが{encoding}ではなかった為、charset-normalizerを使い文字コードの判定を試みましたが失敗しました'
            f'(return empty {type(return_empty_value)}: {path})'
        )
    except FileNotFoundError:
        log.warning(
            '読み込もうとしたファイルが存在しません'
            f'(return empty {type(return_empty_value)}): {path}'
        )
    except PermissionError:
        log.warning(
            '読み込み権限がないか、ファイルへのパスが正しく指定されていません'
            f'(return empty {type(return_empty_value)}): {path}'
        )
    except IsADirectoryError:
        log.warning(
            '読み込もうとしたパスがディレクトリを指しています'
            f'(return empty {type(return_empty_value)}): {path}'
        )
    except OSError:
        log.warning(f'OSで問題が発生しました(return empty {type(return_empty_value)}): {path}')
    except ParserError:
        log.warning(f'ファイルの記述が不正です(return empty {type(return_empty_value)}): {path}')
    else:
        return read_data

    return return_empty_value


def common_file_write_exception_handling(
    func: Callable[[object, IO[Any]], None],
    data: object,
    path: str | PathLike[str],
    *,
    mode: Literal['w', 'wb'] = 'w',
) -> bool:
    """ファイル書き込み時の汎用的な例外処理をするラッパー関数

    Args:
        func (Callable[[object, IO[Any]], None]):
            openしたファイルに対して書き込み処理をする関数
        data (T): 書き込むデータ
        path (str | PathLike[str]): 保存するファイルのパス
        mode (Literal['w', 'wb'], optional): 書き込むファイルをopenする時のmode. Defaults to 'r'.

    Returns:
        bool: 処理の成功失敗の判定のための戻り値
    """
    p = Path(path)

    encoding = 'utf_8'
    newline = '\n'
    if p.suffix in {'.bat', '.cmd'}:
        encoding = 'cp932'
        newline = '\r\n'
    if mode == 'wb':
        encoding = None
        newline = None

    result: bool = False

    try:
        p.parent.mkdir(parents=True, exist_ok=True)
        with Path(p).open(mode=mode, encoding=encoding, newline=newline) as f:
            func(data, f)
    except PermissionError:
        log.warning(f'書き込み権限がないか、ファイルへのパスが正しく指定されていません: {path}')
    except IsADirectoryError:
        log.warning(f'書き込もうとしたパスがディレクトリを指しています: {path}')
    except OSError:
        log.warning(f'OSで問題が発生しました: {path}')
    else:
        result = True

    return result



import json
from logging import getLogger
from os import PathLike
from typing import IO, Any

from abara_file_io.common_io_wrapper import (
    common_file_read_exception_handling,
    common_file_write_exception_handling,
)

log = getLogger(__name__)


def read_json(path: str | PathLike[str]) -> dict:
    """jsonファイルを読み込む

    Args:
        path (str): 読み込むjsonファイルのパス

    Returns:
        dict: 辞書
    """

    def read_json_core(
        f: IO[Any],
    ) -> dict:
        return json.load(f)

    return common_file_read_exception_handling(
        func=read_json_core, return_empty_value={}, path=path
    )


def write_json(data: dict, path: str | PathLike[str], *, ensure_ascii: bool = False) -> bool:
    r"""jsonファイルを書き込む

    Args:
        data (dict): jsonに書き込む辞書オブジェクト
        path (str | PathLike): 保存するファイルパス、ファイル名の拡張子まで記入
        ensure_ascii (bool): 非ASCII文字をエスケープする('あ'→'\\u3042')

    Returns:
        bool: ファイルの保存に成功したらTrue、失敗したらFalse
    """

    def write_json_core(
        data: object,
        f: IO[Any],
    ) -> None:
        json.dump(data, f, indent=2, ensure_ascii=ensure_ascii)

    return common_file_write_exception_handling(func=write_json_core, data=data, path=path)



from logging import getLogger
from os import PathLike
from typing import IO, Any

from abara_file_io.common_io_wrapper import (
    common_file_read_exception_handling,
    common_file_write_exception_handling,
)

log = getLogger(__name__)


def read_text(path: str | PathLike[str]) -> str:
    """テキスト形式のファイルをstrとして読み込む

    UTF-8以外のファイルはchardetで文字コードを自動判定する

    Args:
        path (Path | str): 開くファイルのパス

    Returns:
        str: 読み込んだ文字列、もしファイルが読み込めない場合は空文字列を返す
    """

    def read_text_core(f: IO[Any]) -> str:
        return f.read()

    return common_file_read_exception_handling(
        func=read_text_core,
        return_empty_value='',
        path=path,
    )


def write_text(data: str, path: str | PathLike[str]) -> bool:
    r"""strデータをファイルを書き込む

    テキストファイルを標準的な UTF-8 + \n の形式で保存する
    ただし拡張子.batと.cmdを指定した場合のみ、例外として Shift-JIS + \r\n で保存する

    Args:
        data (str): 書き込む文字列データ
        path (str | PathLike[str]): 保存するファイルのパス（拡張子まで記述）

    Returns:
        bool: ファイルの保存に成功したらTrue、失敗したらFalse
    """

    def write_text_core(
        data: object,
        f: IO[Any],
    ) -> None:
        f.write(data)

    return common_file_write_exception_handling(func=write_text_core, data=data, path=path)



from logging import getLogger
from pathlib import Path

import pytest

from abara_file_io import read_ini, write_ini

log = getLogger(__name__)


@pytest.mark.parametrize(
    ('sample_dicts'),
    [
        pytest.param(1, id='flat_dict'),
        pytest.param(
            2,
            id='section_dict',
        ),
        pytest.param(
            3,
            id='error_dict',
        ),
        pytest.param(
            4,
            id='empty_dict',
        ),
    ],
    indirect=['sample_dicts'],
)
def test_write_ini(
    sample_dicts: dict[str, dict | str],
    tmp_path: Path,
) -> None:
    file_path = tmp_path / 'tmp' / f'test_ini_file_{sample_dicts["name"]}.ini'

    data = {}
    if isinstance(sample_dicts['data'], dict):
        data = sample_dicts['data']

    result = write_ini(data, file_path)

    assert result == sample_dicts['ini_expected']


@pytest.mark.parametrize(
    ('sample_dicts'),
    [
        pytest.param(1, id='flat_dict'),
        pytest.param(
            2,
            id='section_dict',
        ),
        pytest.param(
            3,
            id='error_dict',
        ),
        pytest.param(
            4,
            id='empty_dict',
        ),
    ],
    indirect=['sample_dicts'],
)
def test_read_ini(
    sample_dicts: dict[str, dict | str],
    tmp_path: Path,
) -> None:
    file_path = tmp_path / 'tmp' / f'test_ini_file_{sample_dicts["name"]}.ini'

    data = {}
    if isinstance(sample_dicts['data'], dict):
        data = sample_dicts['data']

    write_ini(data, file_path)
    read_data = read_ini(file_path)

    if sample_dicts['ini_expected']:
        assert data == read_data
    else:
        assert read_data == {}



from logging import getLogger
from pathlib import Path

import pytest

from abara_file_io import read_json, write_json

log = getLogger(__name__)


@pytest.mark.parametrize(
    ('sample_dicts'),
    [
        pytest.param(1, id='flat_dict'),
        pytest.param(
            2,
            id='section_dict1',
        ),
        pytest.param(
            3,
            id='section_dict2',
        ),
        pytest.param(
            4,
            id='empty_dict',
        ),
    ],
    indirect=['sample_dicts'],
)
def test_json_read_write(
    sample_dicts: dict[str, dict | str | bool],
    tmp_path: Path,
) -> None:
    file_path = tmp_path / 'tmp' / f'{sample_dicts["name"]}.json'

    data = {}
    if isinstance(sample_dicts['data'], dict):
        data = sample_dicts['data']

    write_json(data=data, path=file_path)
    response = read_json(path=file_path)

    assert file_path.exists()
    assert response == data



from logging import getLogger
from pathlib import Path

import pytest

from abara_file_io import read_toml, write_toml

log = getLogger(__name__)


@pytest.mark.parametrize(
    ('sample_dicts'),
    [
        pytest.param(1, id='flat_dict'),
        pytest.param(
            2,
            id='section_dict1',
        ),
        pytest.param(
            3,
            id='section_dict2',
        ),
        pytest.param(
            4,
            id='empty_dict',
        ),
    ],
    indirect=['sample_dicts'],
)
def test_write_toml(
    sample_dicts: dict[str, dict | str | bool],
    tmp_path: Path,
) -> None:
    file_path = tmp_path / 'tmp' / f'test_toml_file_{sample_dicts["name"]}.toml'

    data = {}
    if isinstance(sample_dicts['data'], dict):
        data = sample_dicts['data']

    write_toml(data, file_path)

    assert Path(file_path).exists()
    assert read_toml(file_path) == data
